searchData={"items":[{"type":"module","title":"Domainex","doc":"A `domainex` is an Elixir library which provides:\n- common elixir typespec models\n- common domain typespec models\n- a helper for functional dependencies\n- a helper for common domain value objects\n- a helper for common domain needs","ref":"Domainex.html"},{"type":"type","title":"Domainex.aggregate/0","doc":"An `aggregate` in DDD actually is a cluster of *objects*, it possible to be a single entity, or a group\nof entities. The definition of *object* here is just a simple `struct()`.\n\nThis type of `aggregate()`, already set the `aggregate_payload()` to using `DomainEx.Aggregate.Structure`\nas main data structure.","ref":"Domainex.html#t:aggregate/0"},{"type":"type","title":"Domainex.aggregate_name/0","doc":"An `aggregate_name` used to define the business needs behind an `aggregate`.","ref":"Domainex.html#t:aggregate_name/0"},{"type":"type","title":"Domainex.aggregate_payload/0","doc":"An `aggregate_payload()` should be a","ref":"Domainex.html#t:aggregate_payload/0"},{"type":"type","title":"Domainex.error/0","doc":"Common elixir error type spec actually looks like:\n\n```elixir\n  {:error, \"error message\"}\n```\n\nI'm agree with the structure, but I think we need more detail and explicit\nfor the error type. What kind of errors? Is it internal? Is it from core domain?\nIs it from application's level? Or is it an exception?\n\nThe `error_type()` has two possible types:\n- String.t()\n- atom()\n\nBy providing the `error_type()` we can provides more explicit information about\nan error, and help us to provide better error handling based on the error's types\n\nFor the `error_payload()` , it has three possible types\n- String.t()\n- struct()\n- map()\n\nIt is possible to just using a simple string for our error message, but I think sometimes\nwe also need to provide rich error informations, maybe by providing some structs or a map,\nwhich contains specific application or business error handling, such as for application status\ncodes with their definition/message.","ref":"Domainex.html#t:error/0"},{"type":"type","title":"Domainex.event/0","doc":"An `event()` will used at aggregate. An aggregate will trigger / emit\nan event for each domain activity already processed.","ref":"Domainex.html#t:event/0"},{"type":"type","title":"Domainex.event_name/0","doc":"An `event_name()` used to give a name to some event","ref":"Domainex.html#t:event_name/0"},{"type":"type","title":"Domainex.event_payload/0","doc":"An `event_payload()` used for payload for some event, it can\nbe a `struct()` or `map()`.","ref":"Domainex.html#t:event_payload/0"},{"type":"type","title":"Domainex.result/0","doc":"A `result()` actually follow Rust's convention from their `Result `. A `result()`\nis a return value which only has two possibilities a `success()` or `error()` values.\n\nIt's common in Elixir's function which provides two possible return values and looks like this\n\n```elixir\n@spec do_something(a :: atom()) :: {:ok, any()} | {:error, any()}\ndef do_something(a) do\nend\n```\n\nTo simplify the function's signature, I make it to:\n\n```elixir\n@spec do_something(a :: atom()) :: Domainex.result()\ndef do_something(a) do\nend\n```","ref":"Domainex.html#t:result/0"},{"type":"type","title":"Domainex.success/0","doc":"`success()` actually is a common Elixir's convention result for the success return values. It just\nfollow common convention\n\n```elixir\n  {:ok, any()}\n```","ref":"Domainex.html#t:success/0"},{"type":"module","title":"Domainex.Aggregate","doc":"Aggregate is a module provide base Aggregate module functions.\nIt provide base structure for the aggregate. An aggregate itself\nmeans, a cluster of objects that treat as a single unit of domain business\nlogic, although it's possible too to contain only a single object.\n\nThis module should not provide any functions that possible to limiting\nscope of some domain business, or it should be designed to be generic,\nand only provide some helpers","ref":"Domainex.Aggregate.html"},{"type":"function","title":"Domainex.Aggregate.add_event/2","doc":"`add_event/2` used to adding an event to current available events in some aggregate","ref":"Domainex.Aggregate.html#add_event/2"},{"type":"function","title":"Domainex.Aggregate.aggregate/1","doc":"`aggregate/1` used to extract a main aggregate data structure from the tuple of:\n\n```elixir\n  # used to extract structure\n  {:aggregate, {name, structure}}\n```\n\nIt will return an error of `:aggregate` if given parameter is not tuple","ref":"Domainex.Aggregate.html#aggregate/1"},{"type":"function","title":"Domainex.Aggregate.emit_events/1","doc":"`emit_events/1` used to emit all current available events from an aggregate. All of available\nevent will send to all registered processors, and after emitting events, we need to reset current\nevents to empty list.","ref":"Domainex.Aggregate.html#emit_events/1"},{"type":"function","title":"Domainex.Aggregate.entities/1","doc":"`entities/1` used to load multiple entities. It just using `entity/1` under the hood.\nIf your current aggregate contains multiple entities, then it will all of that entities\nin `map()` format.","ref":"Domainex.Aggregate.html#entities/1"},{"type":"function","title":"Domainex.Aggregate.entity/1","doc":"`entity/1` used to load current aggregate's `:contains` property, which expected result\nis a single entity.","ref":"Domainex.Aggregate.html#entity/1"},{"type":"function","title":"Domainex.Aggregate.error_invalid_aggregate_type/0","doc":"","ref":"Domainex.Aggregate.html#error_invalid_aggregate_type/0"},{"type":"function","title":"Domainex.Aggregate.error_invalid_data_type/0","doc":"","ref":"Domainex.Aggregate.html#error_invalid_data_type/0"},{"type":"function","title":"Domainex.Aggregate.is_aggregate?/1","doc":"`is_aggregate?/1` used to check if given tuple is an `:aggregate` or not. For any types\nwhich not a tuple, it will return false","ref":"Domainex.Aggregate.html#is_aggregate?/1"},{"type":"function","title":"Domainex.Aggregate.new/3","doc":"A `new/2` has two possibilities depends on given parameter.\nIf it give a single entity which is a `struct()` it will generate a\n`BaseType.aggregate()` that contains a main aggregate object with single entity,\nor people common said as aggregate root.\n\nIf it give a list of entities, it will generate a `BaseType.aggregate()` that\ncontains a list of entities.\n\nAll generated structure will always generated with an empty `:events`","ref":"Domainex.Aggregate.html#new/3"},{"type":"function","title":"Domainex.Aggregate.update_entity/2","doc":"`update_entity/2` used to update internal aggregate's entity. This function used\nonly for an aggregate with a single entity.\n\nUsage:\n\n  ```elixir\n    aggregate2 = aggregate |> Aggregate.update_entity(fake_entity_updated)\n  ```","ref":"Domainex.Aggregate.html#update_entity/2"},{"type":"function","title":"Domainex.Aggregate.update_entity/3","doc":"`update_entity/3` used to update one of available entities. This function used for an aggregate\nwith multiple entities.\n\nUsage:\n\n  ```elixir\n    {:ok, aggregate2} = aggregate |> Aggregate.update_entity(:fake1, fake_entity_1_updated)\n  ```\n\nThe `update_entity/3` need a `key` to update some entity, since an aggregate that contains multiple\nentities will be mapped based on some unique key","ref":"Domainex.Aggregate.html#update_entity/3"},{"type":"module","title":"Domainex.Aggregate.Structure","doc":"An aggregate objects may using this structure as their\nbase data structure, although the function implementation it\nwill always depends to some specific requirements and logics.\n\nThis structure provides only four possible keys, which are:\n- :name\n- :contains\n- :events\n- :processors\n\nAn aggregate may contains a single entity object or a group of entities.\nAn aggregate also should responsible to emit an event for each domain activities\n\nSpecific for `:processor`, it must be a module which implement behaviour `Event.Processor`","ref":"Domainex.Aggregate.Structure.html"},{"type":"type","title":"Domainex.Aggregate.Structure.t/0","doc":"","ref":"Domainex.Aggregate.Structure.html#t:t/0"},{"type":"module","title":"Domainex.Common","doc":"","ref":"Domainex.Common.html"},{"type":"function","title":"Domainex.Common.extract_element_from_tuple/2","doc":"`extract_element_from_tuple/2` used for extract some element from a tuple. This function already\ncover some exceptions using `try/rescue` mechanism. If some exceptions was raised it will be catched\nand return an error. It use `elem/2` under the hood.","ref":"Domainex.Common.html#extract_element_from_tuple/2"},{"type":"function","title":"Domainex.Common.is_tuple_has_context?/2","doc":"`is_tuple_has_context?/2` is a function to check the first element of some tuple. The *context*\ndefinition here is like an `:ok` or an `:error` or anything that define the tuple's value itself,\nsuch for a success return value, the first element should be an `:ok`.","ref":"Domainex.Common.html#is_tuple_has_context?/2"},{"type":"function","title":"Domainex.Common.is_tuple_length_valid?/2","doc":"`is_tuple_length_valid?/2` is a function to check tuple's length","ref":"Domainex.Common.html#is_tuple_length_valid?/2"},{"type":"module","title":"Domainex.Event","doc":"Event described on this module is specified for domain's event.\nIt provides a base event's structure and also a behaviour to handle\nall available events.","ref":"Domainex.Event.html"},{"type":"function","title":"Domainex.Event.error_invalid_event_type/0","doc":"","ref":"Domainex.Event.html#error_invalid_event_type/0"},{"type":"function","title":"Domainex.Event.is_event?/1","doc":"`is_event?/1` used to check if given tuple is an event or not","ref":"Domainex.Event.html#is_event?/1"},{"type":"function","title":"Domainex.Event.new/2","doc":"`new/2` used to generate new Event structure with given `name` and `payload`. The `payload` itself\nmay be a `struct()` or `map()` depends on business logic needs.","ref":"Domainex.Event.html#new/2"},{"type":"function","title":"Domainex.Event.payload/1","doc":"`payload/1` used to extract event's payload from given event's tuple","ref":"Domainex.Event.html#payload/1"},{"type":"function","title":"Domainex.Event.structure/1","doc":"`structure/1` used to extract base event's structure","ref":"Domainex.Event.html#structure/1"},{"type":"behaviour","title":"Domainex.Event.Processor","doc":"A `Event.Processor` is an abstraction interface to process all emitted events from an aggregate.\nI think it should be better to provide just a behaviour with a simple callback function and give\nthe freedom back to the caller for the detail how to manage all available events from an aggregate,\nmaybe using something like `GenStage` or others, depends on their business needs.\n\nEach of module implement this behaviour should be registered on aggregate, following pattern `Observer`.\nEach time aggregate emit all of available events, it will use registered event's processor and send all\nevents to its callback function, which is `process`.  The module may process all of events async or sync\nand give the result back to the aggregate.","ref":"Domainex.Event.Processor.html"},{"type":"callback","title":"Domainex.Event.Processor.process/1","doc":"`process/1` will got a list of available events from an aggregate, and will return `none()`. It's\nfree for each module implementer how to manage all of these events.  The `:aggregate` will not care\nif it will be an `:ok` or an `:error`, this function should not return anything, or in other languages\nit called as `void`, in Elixir I'm choosing `none()`.","ref":"Domainex.Event.Processor.html#c:process/1"},{"type":"module","title":"Domainex.Event.Structure","doc":"Event.Structure used to grouping all necessary properties for the\ndomain event, which including:\n\n- name\n- payload\n- timestamp\n\nBy using this structure we can simplify our base type spec for event, it\nwill become like this:\n\n  ```elixir\n    {:event, Event.Structure.t()}\n  ```\n\nCompare with previous implementation\n\n  ```elixir\n    {:event, {event_name(), event_payload()}}\n  ```\n\nIt become more simpler","ref":"Domainex.Event.Structure.html"},{"type":"type","title":"Domainex.Event.Structure.t/0","doc":"","ref":"Domainex.Event.Structure.html#t:t/0"},{"type":"extras","title":"Domainex","doc":"# Domainex\n\n`DomainEx` is an Elixir library which provides a set of common typespec and domain models and also provides\na set of function helpers for basic function and domain building  \n\n# About Domainex","ref":"readme.html"},{"type":"extras","title":"Why TypeSpces - Domainex","doc":"Elixir is not a static typing language, it's dynamic typing, which mean when we doesn't need to define any variable\nor function type parameters. But Elixir provides their `TypeSpecs` that really useful to: \n\n- Documentation. I'm one of believer that good (and beautiful) code documentation is important\n- Code analysis using `Dialyzer`\n\nWhen I begin to learn Elixir's *typespec* , I'm starting to learn the mental model, and I really like it. The typespec\nactually is *just* a *typehint* , but somehow I've felt that the type specification mechanism still able to help us to provide rich modeling domain business and in the same time can help us to building a great domain business documentation\nfrom our codes.\n\nThe `Domainex` provides common types such as: \n\n```elixir\n  @type error :: {:error, {error_type(), error_payload()}}\n  @type success :: {:ok, any()}\n  @type result :: success() | error()\n```","ref":"readme.html#why-typespces"},{"type":"extras","title":"Domain Driven Design - Domainex","doc":"Although Elixir is not a static type language, we are still possible to modeling business needs by take a leverage of *typespec*.\n\n`Domainex` also build with purpose to provide a helpers and also *specs* to define some common DDD concepts.\n\n#","ref":"readme.html#domain-driven-design"},{"type":"extras","title":"Aggregate - Domainex","doc":"```elixir\n  @type aggregate_name :: String.t() | atom()\n  @type aggregate_payload :: struct() | map()\n  @type aggregate :: {:aggregate, Aggregate.Structure.t()}\n```\n\nThe main aggregate's structure will be like this: \n\n```elixir\n    @enforce_keys [:name, :contains, :events, :processors]\n    defstruct [:name, :contains, :events, :processors]\n\n    @type t :: %__MODULE__{\n      name: BaseType.aggregate_name(),\n      contains: BaseType.aggregate_payload() | %{atom() => BaseType.aggregate_payload()},\n      events: list(BaseType.event()),\n      processors: list(module())\n    }\n```\n\nInitiate new aggregate: \n\n```elixir\n    fake_entity = %FakeEntityStruct{name: \"fake_entity\"}\n    aggregate = Aggregate.new(:fake_entity, fake_entity, [FakeEventProcessor])\n```\n\nInitiate new aggregate with multiple entities:\n\n```elixir\n    fake_entity_1 = %FakeEntityStruct{name: \"fake_entity_1\"}\n    fake_entity_2 = %FakeEntityStruct{name: \"fake_entity_2\"}\n    aggregate = Aggregate.new(:fake_agg, %{:fake1 => fake_entity_1, :fake2 => fake_entity_2}, [FakeEventProcessor])\n```\n\n##","ref":"readme.html#aggregate"},{"type":"extras","title":"Aggregate and Functional Programming - Domainex","doc":"We should treat an `aggregate` as a *single unit* of business domain, which mean, we should put our business logic inside an `aggregate`. It is a common to create an *object* which in OOP will be a *class* that hold refences to its internal states and behaviors based on business needs. There is a chance that some of aggregates need some common states,properties or even activities, maybe something like emitting domain event. All of these common properties and behaviors can be grouped into some *base aggregate* which will be inherited by other child aggregates. \n\nThe problem is, I rarely see such a thing in functional languages, including in Elixir. There is no way to *extend* from some defined *structure* or a `struct()`.\n\nIn functional, actually it help us to made all things becomes more simpler. There are no *internal states*, no *inheritance*. There are just an input parameters and a functions. An input parameter is just a *value* , and a function used to do some computation, transform a *value* to other *value*.\n\n```\n|-------|            |-----------|        |---------|\n| input | ---------> |  function |------->|  output |\n|-------|            |-----------|        |---------|\n```\n\nEven better, each of *value* is also *immutable* , so there is no chance that we can *update* the value directly, what we can do is create another new value based on some given values.\n\nIf we can't *extend* this `aggregate()` type and structure, then how do we use it in our real application, real business needs ?\n\nThere is no chance to *extend*, **but**, we can use this `DomainEx.Aggregate.Structure` as a *value*, and use all of available functions form `Domainex.Aggregate` as a helper functions, as long as the input value following spec: \n\n```elixir\n  @type aggregate :: {:aggregate, Aggregate.Structure.t()}\n```\n\nYou are still free to create your own `aggregate` based on your business needs, `Domainex` will not limiting the solution or force you to follow some rules.\n\nExample of possible solutions : \n\n```elixir\n  defmodule My.Aggregate do\n    defmodule Structure do\n      # The `:base` property defined here used to store our `aggregate()` type\n      defstruct [:base, :field1, :field2]\n    end\n\n    @spec new(base :: Domainex.aggregate())\n    def new(base) do\n      %Structure{\n        base: base\n      }\n    end\n\n    @spec your_business_activity(structure :: Structure.t()) :: {:ok, term()} | {:error, term()}\n    def your_business_activity(structure) do\n      # do whatever you needs\n    end\n  end\n```\n\nSo, the logic flow become like this: \n\n```mermaid\nflowchart TD;\n  Start-->CreateBaseAggregate\n  CreateBaseAggregate-->|inject| CreateYourAggregate\n  CreateYourAggregate-->|result| YourOwnAggregate\n```\n\n> Please remember, that since the `Domainex.Aggregate` which act as base aggregate used as a *value*, it needs to always be passed as a function parameters. \n\n#","ref":"readme.html#aggregate-and-functional-programming"},{"type":"extras","title":"Domain Event - Domainex","doc":"```elixir\n  @type event_name :: atom()\n  @type event_payload :: struct() | map()\n  @type event :: {:event, Event.Structure.t()}\n```\n\nThe main event's structure will be like this: \n\n```elixir\n    @enforce_keys [:name, :payload, :timestamp]\n    defstruct [:name, :payload, :timestamp]\n\n    @type t :: %__MODULE__{\n      name: BaseType.event_name(),\n      payload: BaseType.event_payload(),\n      timestamp: DateTime.t()\n    }\n```\n\nBy default you almost doesn't need to do anything with these domain event structure and even its *specs*. When you\ninitiate a new aggregate, it will also initiate an empty events.\n\nAggregate and domain events will follow `Observer` design pattern. When you initiate an aggregate, you need to register\nsome event's processor, like an example above.  An `Event.Processor` is a Elixir's *behaviour*, or an *interface* called in other languages.\n\nEach time you `emit_events/1` from an aggregate, it will send all available aggregate's event to its event's processor. Its up to the module which implement `Event.Processor` to do anything with given event list, maybe doing some computation using Elixir's `GenStage`.","ref":"readme.html#domain-event"},{"type":"extras","title":"The power of Tuple - Domainex","doc":"When I learning Elixir, I've seen a lot of *tuple* used to grouping some context. Let just take our previous sample for return values : \n\n```elixir\n  @type error :: {:error, {error_type(), error_payload()}}\n  @type success :: {:ok, any()}\n  @type result :: success() | error()\n```\n\nIn the first time, it just look *weird*, but after learn more, I just think that it actually a reall simple and powerfull concept. We can use *tuple* to build a set of context from some value, not just its *type* but also the *context*, what kind of information do we get from some value. From the example above, when we got the information that the *value* is a *succes* or an *error*, we know how to handling it.\n\nIt's same with previous *aggregate*. By just defining an *aggregate* as a *tuple* , when we got a *value* which is a *tuple* and the first element is `:aggregate`, what we need to do next is extract the following elements like for `aggregate_name` and it's payload.\n\nAnd thanks to Elixir's pattern matching its really simple to *match* and *extract* tuple values\n\n```elixir\niex(1)> {typed, value} = {:ok, \"hello world\"}\n{:ok, \"hello world\"}\niex(2)> typed\n:ok\niex(3)> value\n\"hello world\"\niex(4)> \n```\n\n# Installation\n\nIf [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `domainex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:domainex, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#the-power-of-tuple"}],"content_type":"text/markdown"}